//////////////////////////////////////////////////
//////////////////////////////////////////////////
// This source code is Copyright (2011) of      //
//  Jeppe R. Andersen and Jennifer M. Smillie   //
// and is distributed under the                 //
// Gnu Public License version 2                 //
// http://www.gnu.org/licenses/gpl-2.0.html     //
// You are allowed to distribute and alter the  //
// source under the conditions of the GPLv2     //
// as long as this copyright notice             //
// is unaltered and distributed with the source //
// Any use should comply with the               //
//             MCNET GUIDELINES                 //
//    for Event Generator Authors and Users     //
// as distributed with this source code         //
//////////////////////////////////////////////////
//////////////////////////////////////////////////
#ifndef _GENERATORFLAGS_H_INCLUDED_
#define _GENERATORFLAGS_H_INCLUDED_
#include "fastjet/PseudoJet.hh"
#include "fastjet/ClusterSequence.hh"

struct SGeneratorFlags {
  SGeneratorFlags() {
    beam[0]=1; // Negative z-direction 1: proton -1:anti-proton
    beam[1]=1; // Positive z-direction 1: proton -1:anti-proton
    Ebeam=7000.; // beam energy in GeV
    mstwpdffile="Grids/mstw2008nlo.00.dat";
    // LHAPDF setup:
    LHAPDFName="MSTW2008nlo90cl";
    LHAPDFsubset=0;
    useLHApdf=false;
    raninitfile="";
    LHoutput=false;
    LHoutputfile="LHEvents.dat";
    rootfile="output.root";
    unweight=false;
    scale=40.;
    resum=true;
    match=false;
    nonFKLmatch2j=false;
    nonFKLmatch3j=false;
    nonFKLmatch4j=false;
    rootevents=false;
    NEvents=10000000;
    jetptmin=40.;
    extpartonptmin=35.;
    partonptmax=7000.;
    MW=80.419; // The W mass in GeV/c^2
    GammaW=2.04759951; // the W width in GeV/c^2
    pptmin=15.; // Minimum pt of particle from W-decay
    pptmax=7000.; // Maximym pt of the particle from the W-decay
    scalesetting=0;
    scalefactor=1.;  // Default is *1 for scale factor in PTMAX mode
    /**set fastjet flags */
    logcorrect=0;
    Rparam = 0.6;
    fastjet::Strategy strategy = fastjet::Best;
    fastjet::RecombinationScheme recomb_scheme = fastjet::E_scheme;
    jetalgo=fastjet::antikt_algorithm;
    /** Define jet for fastjet*/
    jet_def=fastjet::JetDefinition(jetalgo, Rparam, recomb_scheme, strategy);
  };
  int beam[2];
  double Ebeam;
  string mstwpdffile;
  string LHAPDFName;
  int LHAPDFsubset;
  bool useLHApdf;
  string raninitfile;
  bool LHoutput;
  string LHoutputfile;
  string rootfile;
  bool unweight;
  bool nonFKLmatch2j;
  bool nonFKLmatch3j;
  bool nonFKLmatch4j;
  bool match;
  bool resum;
  /* double matchingscale; */
  bool rootevents;
  double scale;
  unsigned int NEvents;
  double jetptmin;
  double pptmin,pptmax;
  double partonptmax;
  double MW; // The W mass in GeV/c^2
  double GammaW; // the W width in GeV/c^2
  double extpartonptmin;
  int channels; // which decay channels of the W to include? To be specified
  double Rparam;
  int scalesetting; //0:fixed, 1:Ht
  double scalefactor;  // for rescaling when in PTMAX mode
  int logcorrect;  // include extra logs
  fastjet::Strategy strategy;
  fastjet::RecombinationScheme recomb_scheme;
  fastjet::JetAlgorithm jetalgo;/** Define jet for fastjet*/
  fastjet::JetDefinition jet_def;
};
#endif
