//////////////////////////////////////////////////
//////////////////////////////////////////////////
// This source code is Copyright (2011) of      //
//  Jeppe R. Andersen and Jennifer M. Smillie   //
// and is distributed under the                 //
// Gnu Public License version 2                 //
// http://www.gnu.org/licenses/gpl-2.0.html     //
// You are allowed to distribute and alter the  //
// source under the conditions of the GPLv2     //
// as long as this copyright notice             //
// is unaltered and distributed with the source //
// Any use should comply with the               //
//             MCNET GUIDELINES                 //
//    for Event Generator Authors and Users     //
// as distributed with this source code         //
//////////////////////////////////////////////////
//////////////////////////////////////////////////
#include "JetGen.h"

SGeneratorFlags Flags;
double jetrapmax(4.5);
double ptbarmin(0.);  //  Don't apply cut unless you set it by hand
double jetptdelta(0.); // the offset in jet pts

int init(int argn, char* argv[])
{
  ifstream input;
  istringstream sline;
  int iline(0);
  string label,temp,line;

  if (argn!=2) {
    cerr << "# Usage:\n./JetGen [input.file]\n";
    cerr << "# where input.file is a file with parameters controlling the generation of events.\n";
    if (argn>2)
      return 1;
    else {
      cout << "# Using standard values for generator parameters\n";
      return 0;
    }
  } else {
    // Now open input file:
    input.open(argv[1], ios::in); // Open input file
    
    // Read parameters in from input file:
    while(getline(input,line)) {
      line.push_back(' ');
      sline.str(line);
      sline>>label;
      iline++;
      if(label=="Ebeam:") {
	if(!(sline>>Flags.Ebeam)) {
	  cout << "Error in input file at line " <<iline<<endl;
	  cout << "Value of Beam Energy invalid!"<<endl;
	  return 1;
	}
	if(Flags.Ebeam<0) {
	  cout << "Beam Energy<0 at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="jetptmin:") {
	if(!(sline>>Flags.jetptmin)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of jetptmin invalid!"<<endl;
	  return 1;
	}
	if(Flags.jetptmin<0) {
	  cout <<"jetptmin<0 at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="jetptdelta:") {
	if(!(sline>>jetptdelta)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of jetptdelta invalid!"<<endl;
	  return 1;
	}
	if(jetptdelta<0) {
	  cout <<"jetptdelta<0 at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="extpartonptmin:") {
	if(!(sline>>Flags.extpartonptmin)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of extpartonptmin invalid!"<<endl;
	  return 1;
	}
	if(Flags.extpartonptmin<0) {
	  cout <<"extpartonptmin<0 at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="scale:") {
	if(!(sline>>Flags.scale)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of scale invalid!"<<endl;
	  return 1;
	}
	if(Flags.scale<0) {
	  cout<<"scale<0 at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="scalesetting:") { //0: fixed 1:PTMAX 2:geometric mean
	if(!(sline>>Flags.scalesetting)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of scalesetting invalid!"<<endl;
	  return 1;
	}
	if(Flags.scalesetting!=0&&Flags.scalesetting!=1&&Flags.scalesetting!=2) {
	  cout<<"scalesetting \notin {0,1,2}  at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="scalefactor:") {
	if(!(sline>>Flags.scalefactor)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of scalefactor invalid!"<<endl;
	  return 1;
	}
	if(Flags.scalefactor<0) {
	  cout<<"scalefactor <0  at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="logcorrect:") { 
	if(!(sline>>Flags.logcorrect)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of logcorrect invalid!"<<endl;
	  return 1;
	}
	if(Flags.logcorrect!=0&&Flags.logcorrect!=1) {
	  cout<<"logcorrrect \notin {0,1}  at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="jetrapmax:") {
	if(!(sline>>jetrapmax)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of jetrapmax invalid!"<<endl;
	  return 1;
	}
	if(jetrapmax<0) {
	  cout<<"jetrapmax<0 at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if (label=="ptbarmin:") {
	if (!(sline>>ptbarmin)) {
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of ptbarmin invalid!"<<endl;
	  return 1;
	}
	if (ptbarmin<0) {
	  cout<<"ptbarmin<0 at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="NEvents:") {
	if(!(sline>>Flags.NEvents)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of NEvents invalid!"<<endl;
	  return 1;
	}
	if(Flags.NEvents<0) {
	  cout<<"NEvents<0 at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="LHAPDFsubset:") {
	if(!(sline>>Flags.LHAPDFsubset)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of LHAPDFsubset invalid!"<<endl;
	  return 1;
	}
	if(Flags.LHAPDFsubset<0) {
	  cout<<"LHAPDFsubset<0 at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="beam1:") {
	if(!(sline>>temp)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Beam 1 type invalid!"<<endl;
	  return 1;
	}
	if(temp=="P") Flags.beam[0]=1;
	else if(temp=="PBAR") Flags.beam[0]=-1;
	else {
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "Unrecognised beam type for beam 1!"<<endl;
	  return 1;
	}
      }
      else if(label=="beam2:") {
	if(!(sline>>temp)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Beam 2 type invalid!"<<endl;
	  return 1;
	}
	if(temp=="P") Flags.beam[1]=1;
	else if(temp=="PBAR") Flags.beam[1]=-1;
	else {
	  cout <<"Error in input file at line "<<iline<<endl;
	  cout <<"Unrecognised beam type for beam 2!"<<endl;
	  return 1;
	}
      }
      else if(label=="jetalgo:") {
	if(!(sline>>temp)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"jetalgo type invalid!"<<endl;
	  return 1;
	}
	if(temp=="antikt") Flags.jetalgo=fastjet::antikt_algorithm;
	else if(temp=="kt") Flags.jetalgo=fastjet::kt_algorithm;
	else {
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "Unrecognised beam type for beam 1!"<<endl;
	  return 1;
	}
      }
      else if(label=="Rparam:") {
	if(!(sline>>Flags.Rparam)){
	  cout<<"Error in input file at line "<<iline<<endl;
	  cout<<"Value of Rparam invalid!"<<endl;
	  return 1;
	}
	if(Flags.Rparam<0) {
	  cout<<"Rparam<0 at line "<<iline<<" of input file!"<<endl;
	  return 1;
	}
      }
      else if(label=="LHoutput:") {
	if(!(sline>>temp)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "LHoutput flag invalid!"<<endl;
	  return 1;
	}
	if(temp=="Y"||temp=="y") Flags.LHoutput=true; else Flags.LHoutput=false;
      }
      else if(label=="LHoutputfile:") {
	if(!(sline>>Flags.LHoutputfile)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "Eventfile name invalid!"<<endl;
	  return 1;
	}
      }
      else if(label=="rootfile:") {
	if(!(sline>>Flags.rootfile)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "rootfile name invalid!"<<endl;
	  return 1;
	}
      }
      else if(label=="mstwpdffile:") {
	if(!(sline>>Flags.mstwpdffile)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "mstwpdffile name invalid!"<<endl;
	  return 1;
	}
      }
      else if(label=="LHAPDFName:") {
	if(!(sline>>Flags.LHAPDFName)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "LHAPDFName name invalid!"<<endl;
	  return 1;
	}
      }
      else if(label=="match:") {
	if(!(sline>>temp)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "match flag invalid!"<<endl;
	  return 1;
	}
	if(temp=="Y"||temp=="y") Flags.match=true; else Flags.match=false;
      }
      else if(label=="useLHApdf:") {
	if(!(sline>>temp)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "useLHApdf flag invalid!"<<endl;
	  return 1;
	}
	if(temp=="Y"||temp=="y") Flags.useLHApdf=true; else Flags.useLHApdf=false;
      }
      else if(label=="resum:") {
	if(!(sline>>temp)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "match flag invalid!"<<endl;
	  return 1;
	}
	if(temp=="Y"||temp=="y") Flags.resum=true; else Flags.resum=false;
      }
      else if(label=="nonFKLmatch2j:") {
	if(!(sline>>temp)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "match flag invalid!"<<endl;
	  return 1;
	}
	if(temp=="Y"||temp=="y") Flags.nonFKLmatch2j=true; else Flags.nonFKLmatch2j=false;
      }
      else if(label=="nonFKLmatch3j:") {
	if(!(sline>>temp)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "match flag invalid!"<<endl;
	  return 1;
	}
	if(temp=="Y"||temp=="y") Flags.nonFKLmatch3j=true; else Flags.nonFKLmatch3j=false;
      }
      else if(label=="nonFKLmatch4j:") {
	if(!(sline>>temp)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "match flag invalid!"<<endl;
	  return 1;
	}
	if(temp=="Y"||temp=="y") Flags.nonFKLmatch4j=true; else Flags.nonFKLmatch4j=false;
      }
      else if(label=="rootevents:") {
      	if(!(sline>>temp)){
      	  cout << "Error in input file at line "<<iline<<endl;
      	  cout << "rootevents flag invalid!"<<endl;
      	  return 1;
      	}
      	if(temp=="Y"||temp=="y") Flags.rootevents=true; else Flags.rootevents=false;
      }
      else if(label=="unweight:") {
	if(!(sline>>temp)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "unweight flag invalid!"<<endl;
	  return 1;
	}
	if(temp=="Y"||temp=="y") Flags.unweight=true; else Flags.unweight=false;
      }
      else if(label=="raninitfile:") {
	if(!(sline>>Flags.raninitfile)){
	  cout << "Error in input file at line "<<iline<<endl;
	  cout << "raninitfile name invalid!"<<endl;
	  return 1;
	}
      }
      else if(label.at(0)==47){
      }
      else{
	cout<<"Unrecognised parameter at line "<<iline<<" in input file:"<<endl;
	cout<<line<<endl;
	return 1;
      }
    }
    
    input.close();
  }
  fastjet::Strategy strategy = fastjet::Best;
  fastjet::RecombinationScheme recomb_scheme = fastjet::E_scheme;
  Flags.jet_def=fastjet::JetDefinition(Flags.jetalgo, Flags.Rparam, recomb_scheme, strategy);

  return 0;
}

void Fill2DHistograms(TH1D* h, TH1D* hN, TH1D *hwt,double x,double wt)
{
  double binwidth; // the binwidth
  binwidth=h->GetXaxis()->GetBinWidth(1);
  h->Fill(x,wt/binwidth);
  hwt->Fill(x,wt*wt/binwidth);
  hN->Fill(x);
}

int main(int argn, char *argv[])
{
  double wt,wts; // The event weight
  double ya,yb,yi;
  CLHEP::HepLorentzVector pH,pi;
  int np; // The number of final state partons
  int njetsb,njetst; // The number of final state jets
  CLHEP::HepLorentzVector ja,jb,jthird,jfourth,jha,jhb;
  CLHEP::Ranlux64Engine rUnweight;
  
  // Float_t rpp[24][4]; // four vectors for parton momenta, for use with ROOT
  // Float_t rpH[4]; // the four vector for the Higgs boson, for use with ROOT

  if ((init(argn,argv))==1) 
    return(-1);
  // Only simple analysis and self-consistency check:
  // Example histogram file:
  TFile hfile(Flags.rootfile.c_str(),"RECREATE","Output from JetGen");
  TROOT simple("JetGen","Output from JetGen");
  TH1D *hmstatus,*hwt,*hwtwt;
  TH1D *htotal,*htotalN,*htotalwt2;
  htotal = new TH1D("total","total",1,-0.5,.5);
  htotal->Sumw2();
  htotalN = new TH1D("totalN","totalN",1,-0.5,.5);
  htotalwt2 = new TH1D("totalwt2","totalwt2",1,-0.5,.5);

  TH1D *hya,*hyaN,*hyawt2;
  hya = new TH1D("ya","ya",40,-5.,5.);
  hya->Sumw2();
  hyaN = new TH1D("yaN","yaN",40,-5.,5.);
  hyawt2 = new TH1D("yawt2","yawt2",40,-5.,5.);

  TH1D *hyb,*hybN,*hybwt2;
  hyb = new TH1D("yb","yb",40,-5.,5.);
  hyb->Sumw2();
  hybN = new TH1D("ybN","ybN",40,-5.,5.);
  hybwt2 = new TH1D("ybwt2","ybwt2",40,-5.,5.);

  TH1D *hpt1,*hpt1N,*hpt1wt2;
  hpt1 = new TH1D("pt1","pt1",100,0.,1000.);
  hpt1->Sumw2();
  hpt1N = new TH1D("pt1N","pt1N",100,0.,1000.);
  hpt1wt2 = new TH1D("pt1wt2","pt1wt2",100,0.,1000.);

  TH1D *hptp1,*hptp1N,*hptp1wt2;
  hptp1 = new TH1D("ptp1","ptp1",100,0.,100.);
  hptp1->Sumw2();
  hptp1N = new TH1D("ptp1N","ptp1N",100,0.,100.);
  hptp1wt2 = new TH1D("ptp1wt2","ptp1wt2",100,0.,100.);

  TH1D *hptp2,*hptp2N,*hptp2wt2;
  hptp2 = new TH1D("ptp2","ptp2",100,0.,100.);
  hptp2->Sumw2();
  hptp2N = new TH1D("ptp2N","ptp2N",100,0.,100.);
  hptp2wt2 = new TH1D("ptp2wt2","ptp2wt2",100,0.,100.);

  TH1D *hptbar,*hptbarN,*hptbarwt2;
  hptbar = new TH1D("ptbar","ptbar",100,0.,1000.);
  hptbar->Sumw2();
  hptbarN = new TH1D("ptbarN","ptbarN",100,0.,1000.);
  hptbarwt2 = new TH1D("ptbarwt2","ptbarwt2",100,0.,1000.);

  TH1D *hpt2,*hpt2N,*hpt2wt2;
  hpt2 = new TH1D("pt2","pt2",100,0.,1000.);
  hpt2->Sumw2();
  hpt2N = new TH1D("pt2N","pt2N",100,0.,1000.);
  hpt2wt2 = new TH1D("pt2wt2","pt2wt2",100,0.,1000.);

  TH1D *hpth1,*hpth1N,*hpth1wt2;
  hpth1 = new TH1D("pth1","pth1",100,0.,1000.);
  hpth1->Sumw2();
  hpth1N = new TH1D("pth1N","pth1N",100,0.,1000.);
  hpth1wt2 = new TH1D("pth1wt2","pth1wt2",100,0.,1000.);

  TH1D *hpth2,*hpth2N,*hpth2wt2;
  hpth2 = new TH1D("pth2","pth2",100,0.,1000.);
  hpth2->Sumw2();
  hpth2N = new TH1D("pth2N","pth2N",100,0.,1000.);
  hpth2wt2 = new TH1D("pth2wt2","pth2wt2",100,0.,1000.);

  TH1D *hpthr,*hpthrN,*hpthrwt2;
  hpthr = new TH1D("pthr","pthr",100,0.,1.);
  hpthr->Sumw2();
  hpthrN = new TH1D("pthrN","pthrN",100,0.,1.);
  hpthrwt2 = new TH1D("pthrwt2","pthrwt2",100,0.,1.);

  TH1D *hktn;
  hktn = new TH1D("ktn","ktn",100,0.,1000.);
  hktn->Sumw2();

  TH1D *hht,*hhtN,*hhtwt2;
  hht = new TH1D("ht","ht",100,0.,2000.);
  hht->Sumw2();
  hhtN = new TH1D("htN","htN",100,0.,2000.);
  hhtwt2 = new TH1D("htwt2", "htwt2", 100,0.,2000.);

  TH1D *hsij,*hsijN,*hsijwt2;
  hsij = new TH1D("sij","sij",100,0.,1000000.);
  hsij->Sumw2();
  hsijN = new TH1D("sijN","sijN",100,0.,1000000.);
  hsijwt2 = new TH1D("sijwt2", "sijwt2", 100,0.,1000000.);

  TH1D *hycentre;
  hycentre = new TH1D("ycentre","ycentre",40,-5.,5.);
  hycentre->Sumw2();

  TH1D *hnjets,*hnjetsN,*hnjetswt2;
  hnjets = new TH1D("njets","njets dist",10,0.5,10.5);
  hnjets->Sumw2();
  hnjetsN = new TH1D("njetsN","njetsN dist",10,0.5,10.5);
  hnjetswt2 = new TH1D("njetswt2","njetswt2 dist",10,0.5,10.5);

  TH1D *hnp,*hnpN,*hnpwt2;
  hnp = new TH1D("np","np dist",35,-.5,34.5);
  hnp->Sumw2();
  hnpN = new TH1D("npN","npN dist",35,-.5,34.5);
  hnpwt2 = new TH1D("npwt2","npwt2 dist",35,-.5,34.5);

  TH1D *hydij;
  hydij = new TH1D("ydij","ydij",40,-5.,5.);
  hydij->Sumw2();

  TH1D *havgnjets,*havgnjetsN,*havgnjetswt2;
  havgnjets = new TH1D("avgnjets","avgnjets",50,0.0,10.0);
  havgnjets->Sumw2();
  havgnjetsN = new TH1D("avgnjetsN","avgnjetsN",50,0.0,10.0);
  havgnjetswt2 = new TH1D("avgnjetswt2","avgnjetswt2",50,0.0,10.0);

  TH1D *havgnjetsnorm,*havgnjetsnormN,*havgnjetsnormwt2;
  havgnjetsnorm = new TH1D("avgnjetsnorm","avgnjetsnorm",50,0.0,10.0);
  havgnjetsnorm->Sumw2();
  havgnjetsnormN = new TH1D("avgnjetsnormN","avgnjetsnormN",50,0.0,10.0);
  havgnjetsnormwt2 = new TH1D("avgnjetsnormwt2","avgnjetsnormwt2",50,0.0,10.0);

  hmstatus = new TH1D("mstatus","mstatus",25,-.5,24.5);
  hwt = new TH1D("wt","wt",800,-30.,50.);
  hwtwt = new TH1D("wtwt","wtwt",800,-30.,50.);

  TH1D *hydif,*hydifN,*hydifwt2;
  hydif = new TH1D("ydif","ydif",32,0.,8.);
  hydif->Sumw2();
  hydifN = new TH1D("ydifN","ydifN",32,0.,8.);
  hydifwt2 = new TH1D("ydifwt2","ydifwt2",32,0.,8.);

  TH1D *hydifh;//,*hydifhN,*hydifhwt2;
  hydifh = new TH1D("ydifh","ydifh",32,0.,8.);
  hydifh->Sumw2();
  // hydifhN = new TH1D("ydifhN","ydifhN",32,0.,8.);
  // hydifhwt2 = new TH1D("ydifhwt2","ydifhwt2",32,0.,8.);

  TH1D *hydif3,*hydif3N,*hydif3wt2;
  hydif3 = new TH1D("ydif3","ydif3",32,0.,8.);
  hydif3->Sumw2();
  hydif3N = new TH1D("ydif3N","ydif3N",32,0.,8.);
  hydif3wt2 = new TH1D("ydif3wt2","ydif3wt2",32,0.,8.);

  TH1D *hydiftwojet,*hydiftwojetN,*hydiftwojetwt2;
  hydiftwojet = new TH1D("ydiftwojet","ydiftwojet",32,0.,8.);
  hydiftwojet->Sumw2();
  hydiftwojetN = new TH1D("ydiftwojetN","ydiftwojetN",32,0.,8.);
  hydiftwojetwt2 = new TH1D("ydiftwojetwt2","ydiftwojetwt2",32,0.,8.);

  TH1D *hydifhtwojet;//,*hydifhtwojetN,*hydifhtwojetwt2;
  hydifhtwojet = new TH1D("ydifhtwojet","ydifhtwojet",32,0.,8.);
  hydifhtwojet->Sumw2();
  // hydifhtwojetN = new TH1D("ydifhtwojetN","ydifhtwojetN",32,0.,8.);
  // hydifhtwojetwt2 = new TH1D("ydifhtwojetwt2","ydifhtwojetwt2",32,0.,8.);

  TH1D *hydifthreejet,*hydifthreejetN,*hydifthreejetwt2;
  hydifthreejet = new TH1D("ydifthreejet","ydifthreejet",32,0.,8.);
  hydifthreejet->Sumw2();
  hydifthreejetN = new TH1D("ydifthreejetN","ydifthreejetN",32,0.,8.);
  hydifthreejetwt2 = new TH1D("ydifthreejetwt2","ydifthreejetwt2",32,0.,8.);

  TH1D *hydiffourjet,*hydiffourjetN,*hydiffourjetwt2;
  hydiffourjet = new TH1D("ydiffourjet","ydiffourjet",32,0.,8.);
  hydiffourjet->Sumw2();
  hydiffourjetN = new TH1D("ydiffourjetN","ydiffourjetN",32,0.,8.);
  hydiffourjetwt2 = new TH1D("ydiffourjetwt2","ydiffourjetwt2",32,0.,8.);

  TH1D *hydiffivejet,*hydiffivejetN,*hydiffivejetwt2;
  hydiffivejet = new TH1D("ydiffivejet","ydiffivejet",32,0.,8.);
  hydiffivejet->Sumw2();
  hydiffivejetN = new TH1D("ydiffivejetN","ydiffivejetN",32,0.,8.);
  hydiffivejetwt2 = new TH1D("ydiffivejetwt2","ydiffivejetwt2",32,0.,8.);

  TH1D *hydifsixjet,*hydifsixjetN,*hydifsixjetwt2;
  hydifsixjet = new TH1D("ydifsixjet","ydifsixjet",32,0.,8.);
  hydifsixjet->Sumw2();
  hydifsixjetN = new TH1D("ydifsixjetN","ydifsixjetN",32,0.,8.);
  hydifsixjetwt2 = new TH1D("ydifsixjetwt2","ydifsixjetwt2",32,0.,8.);

  TH1D *hydifsevenjet,*hydifsevenjetN,*hydifsevenjetwt2;
  hydifsevenjet = new TH1D("ydifsevenjet","ydifsevenjet",32,0.,8.);
  hydifsevenjet->Sumw2();
  hydifsevenjetN = new TH1D("ydifsevenjetN","ydifsevenjetN",32,0.,8.);
  hydifsevenjetwt2 = new TH1D("ydifsevenjetwt2","ydifsevenjetwt2",32,0.,8.);

  TH1D *hydifeightjet,*hydifeightjetN,*hydifeightjetwt2;
  hydifeightjet = new TH1D("ydifeightjet","ydifeightjet",32,0.,8.);
  hydifeightjet->Sumw2();
  hydifeightjetN = new TH1D("ydifeightjetN","ydifeightjetN",32,0.,8.);
  hydifeightjetwt2 = new TH1D("ydifeightjetwt2","ydifeightjetwt2",32,0.,8.);

  TH1D *hydifninejet,*hydifninejetN,*hydifninejetwt2;
  hydifninejet = new TH1D("ydifninejet","ydifninejet",32,0.,8.);
  hydifninejet->Sumw2();
  hydifninejetN = new TH1D("ydifninejetN","ydifninejetN",32,0.,8.);
  hydifninejetwt2 = new TH1D("ydifninejetwt2","ydifninejetwt2",32,0.,8.);

  TH1D *hcosphi,*hcosphiN,*hcosphiwt2;
  hcosphi = new TH1D("cosphi","Avg. cos(phijj)",52,0.0,13.0);
  hcosphi->Sumw2();
  hcosphiN = new TH1D("cosphiN","Avg. cos(phijj)",52,0.0,13.0);
  hcosphiwt2 = new TH1D("cosphiwt2","Avg. cos(phijj)",52,0.0,13.0);

  TH1D *hcosphinorm,*hcosphinormN,*hcosphinormwt2;
  hcosphinorm = new TH1D("cosphinorm","Avg. cos(phijj)",52,0.0,13.0);
  hcosphinorm->Sumw2();
  hcosphinormN = new TH1D("cosphinormN","Avg. cos(phijj)",52,0.0,13.0);
  hcosphinormwt2 = new TH1D("cosphinormwt2","Avg. cos(phijj)",52,0.0,13.0);

  TH1D *hdsdphi1,*hdsdphi1N,*hdsdphi1wt2;
  hdsdphi1 = new TH1D("dsdphi1","dsigma/dphi",31,0.0,3.1);
  hdsdphi1->Sumw2();
  hdsdphi1N = new TH1D("dsdphi1N","dsigma/dphi",31,0.0,13.0);
  hdsdphi1wt2 = new TH1D("dsdphi1wt2","dsigma/dphi",31,0.0,13.0);

  TH1D *hdsdphi2,*hdsdphi2N,*hdsdphi2wt2;
  hdsdphi2 = new TH1D("dsdphi2","dsigma/dphi",31,0.0,3.1);
  hdsdphi2->Sumw2();
  hdsdphi2N = new TH1D("dsdphi2N","dsigma/dphi",31,0.0,13.0);
  hdsdphi2wt2 = new TH1D("dsdphi2wt2","dsigma/dphi",31,0.0,13.0);

  TH1D *hdsdphi3,*hdsdphi3N,*hdsdphi3wt2;
  hdsdphi3 = new TH1D("dsdphi3","dsigma/dphi",31,0.0,3.1);
  hdsdphi3->Sumw2();
  hdsdphi3N = new TH1D("dsdphi3N","dsigma/dphi",31,0.0,13.0);
  hdsdphi3wt2 = new TH1D("dsdphi3wt2","dsigma/dphi",31,0.0,13.0);

  TH1D *hdsdphi4,*hdsdphi4N,*hdsdphi4wt2;
  hdsdphi4 = new TH1D("dsdphi4","dsigma/dphi",31,0.0,3.1);
  hdsdphi4->Sumw2();
  hdsdphi4N = new TH1D("dsdphi4N","dsigma/dphi",31,0.0,13.0);
  hdsdphi4wt2 = new TH1D("dsdphi4wt2","dsigma/dphi",31,0.0,13.0);

  TProfile * tpavgcosphi;
  tpavgcosphi = new TProfile("tpavgcosphi","Avg. cos phi",52,0.,13.,-1.,1.,"s");
  tpavgcosphi->Sumw2();

  TProfile * tpavgjets;
  tpavgjets = new TProfile("tpavgjets","Avg. jets",35,0.,10.,2.,20.,"s");
  tpavgjets->Sumw2();

  TProfile * tphtjets;
  tphtjets = new TProfile("tphtjets","Avg. HT",100,0.,2000.,2.,20.,"s");
  tphtjets->Sumw2();

  TProfile * tpsijjets;
  tpsijjets = new TProfile("tpsijjets","avg jets vs sij",100,0.,1000000.,2.,20.,"s");
  tpsijjets->Sumw2();

  // Make tree of events:
  Float_t rpp[22][4]; // four vectors for parton momenta, for use with ROOT
  Int_t processtype;
  int nj;
  Double_t weight05,weight2;
  TTree * Events;
  Events = new TTree("Events","Events");
  Events->Branch("weight",&wts,"weights/D");
  Events->Branch("weight05",&weight05,"weights05/D");
  Events->Branch("weight2",&weight2,"weights2/D");
  Events->Branch("nj",&nj,"nj/I");
  Events->Branch("Pfourv",&rpp,"Pfourv[nj][4]/F");
  Events->Branch("processtype",&processtype,"processtype/I");

  CMultijet event(Flags); // use just standard settings

  unsigned int evententries(0);
  
  for (unsigned int i=0;i<Flags.NEvents;i++) {
    wt=event.Event(); // in micro-barn
    //    std::cout<<"WJetGen: "<<wt<<"  "<<event.getMstatus()<<"\n";
    wts=wt/double(Flags.NEvents); // in micro-barn
    weight05=event.getwt05()/double(Flags.NEvents); // in micro-barn
    weight2=event.getwt2()/double(Flags.NEvents); // in micro-barn
    hmstatus->Fill(event.getMstatus());
    if (event.getMstatus()==0) { // If event was good
      // Number of partons in event:
      np=event.getnp();
      
      std::vector<fastjet::PseudoJet> avec;
      
      //       std::cout<<"====================\n";
      for (int ii=0;ii<np;ii++) {
	pi=event.getpartvn(ii);
	// 	cout << pi<<endl;
	avec.push_back(pi);
      }
      //       std::cout<<"--------------------\n";
	
      // pi=event.getpartvn(0);
      // ypa=pi.rapidity();
      // pi=event.getpartvn(np-1);
      // ypb=pi.rapidity();
      
      // /**set fastjet flags */
      // double Rparam = 0.6;
      // fastjet::Strategy strategy = fastjet::Best;
      // fastjet::RecombinationScheme recomb_scheme = fastjet::E_scheme;
      
      // /** Define jet for fastjet*/
      // fastjet::JetDefinition jet_def(fastjet::antikt_algorithm, Rparam, recomb_scheme, strategy);
      
      /** fastjet find clusters */
      fastjet::ClusterSequence ev(avec, Flags.jet_def);
      
      /** Retrieve the final state jets from fastjet Sorted by Rapidity*/
      std::vector<fastjet::PseudoJet> jets = sorted_by_rapidity(ev.inclusive_jets(Flags.jetptmin));
      
      // njets=int(jets.size());
      // Now find the tagging jets
      njetst=0;
      njetsb=0;
      ya=10;
      yb=-10;
      double ht=0.;
      // Count how many of the fastjet jets have Et>energy_cut GeV 
      // and calculate largest distance between jets of more than ptcut2
      std::vector<fastjet::PseudoJet>::const_iterator itr = jets.begin();
      std::vector<fastjet::PseudoJet>::const_iterator jai,jbi;
      for( ; itr != jets.end() ; ++itr) {
	//	if (((*itr).perp()>Flags.jetptmin)&&((fabs((*itr).rapidity())<3.)||((fabs((*itr).rapidity())>3.5)&&(fabs((*itr).rapidity())<4.8)))) {
	if (((*itr).perp()>Flags.jetptmin)&&(fabs((*itr).rapidity())<jetrapmax)) {
	  njetsb++;
	  if ((*itr).perp()>Flags.jetptmin) {
	    njetst++;
	    ht+=(*itr).perp();
	    yi=(*itr).rapidity();
	    if (yi<ya) { ya=yi ; ja.set((*itr)[0],(*itr)[1],(*itr)[2],(*itr)[3]);jai=itr;};
	    if (yi>yb) { yb=yi ; jb.set((*itr)[0],(*itr)[1],(*itr)[2],(*itr)[3]);jbi=itr;};
	  }
	}
      }
      
      double ptbar=(ja.perp()+jb.perp())/2.;
      // the jets sorted by pt
      std::vector<fastjet::PseudoJet> hjets = sorted_by_pt(ev.inclusive_jets(Flags.jetptmin));

      if ((njetst>=2) && (ptbar > ptbarmin) && (hjets.at(0).perp()>=(Flags.jetptmin+jetptdelta)))
	// Perform the analysis
	{
	  event.xsec[event.getprocesstype()][0]+=wts;
	  event.xsec[event.getprocesstype()][1]+=wts*wts;
	  if (wt>event.maxweight[event.getprocesstype()])
	    event.maxweight[event.getprocesstype()]=wt;
	  
	  if (Flags.rootevents) {
	    if ((Flags.unweight&&(rUnweight.flat()<wt/event.maxweight[event.getprocesstype()]))||!Flags.unweight) {
	      int ii=0;
	      std::vector<fastjet::PseudoJet>::const_iterator itr = jets.begin();
	      for( ; itr != jets.end() ; ++itr) {
		rpp[ii][0]=(*itr).e();
		rpp[ii][1]=(*itr).px();
		rpp[ii][2]=(*itr).py();
		rpp[ii][3]=(*itr).pz();
		++ii;
	      }
	      for (int iii=ii;iii<22;iii++) { // reset the rest
		rpp[iii][0]=0.;
		rpp[iii][1]=0.;
		rpp[iii][2]=0.;
		rpp[iii][3]=0.;
	      }
	      processtype=event.getprocesstype();
	      nj=jets.size();
	      Events->Fill();
	      ++evententries;
	    }	      
	    if (evententries % 1000000 == 0) Events->AutoSave();
	  }

	  // std::cout <<pe->perp() << " "<<pe->rapidity()<<" "<<pnu->perp()<<std::endl;
	  // htotal->Fill(0.,wts);
	  Fill2DHistograms(htotal,htotalN,htotalwt2,0.,wts);
	  // hya->Fill(ya,wts*4.);
	  Fill2DHistograms(hya,hyaN,hyawt2,ya,wts);
	  // hyb->Fill(yb,wts*4.);
	  Fill2DHistograms(hyb,hybN,hybwt2,yb,wts);
	  // hydif->Fill(fabs(yb-ya),wts*4.);
	  Fill2DHistograms(hydif,hydifN,hydifwt2,fabs(yb-ya),wts);
	  // hye->Fill(pe->rapidity(),wts*4.);
	  if (njetst>=3)
	    Fill2DHistograms(hydif3,hydif3N,hydif3wt2,fabs(yb-ya),wts);
	  if (njetst==2)
	    Fill2DHistograms(hydiftwojet,hydiftwojetN,hydiftwojetwt2,fabs(yb-ya),wts);
	  if (njetst==3)
	    Fill2DHistograms(hydifthreejet,hydifthreejetN,hydifthreejetwt2,fabs(yb-ya),wts);
	  if (njetst==4)
	    Fill2DHistograms(hydiffourjet,hydiffourjetN,hydiffourjetwt2,fabs(yb-ya),wts);
	  if (njetst==5)
	    Fill2DHistograms(hydiffivejet,hydiffivejetN,hydiffivejetwt2,fabs(yb-ya),wts);
	  if (njetst==6)
	    Fill2DHistograms(hydifsixjet,hydifsixjetN,hydifsixjetwt2,fabs(yb-ya),wts);
	  if (njetst==7)
	    Fill2DHistograms(hydifsevenjet,hydifsevenjetN,hydifsevenjetwt2,fabs(yb-ya),wts);
	  if (njetst==8)
	    Fill2DHistograms(hydifeightjet,hydifeightjetN,hydifeightjetwt2,fabs(yb-ya),wts);
	  if (njetst==9)
	    Fill2DHistograms(hydifninejet,hydifninejetN,hydifninejetwt2,fabs(yb-ya),wts);
	  Fill2DHistograms(hpt1,hpt1N,hpt1wt2,ja.perp(),wts);
	  Fill2DHistograms(hpt2,hpt2N,hpt2wt2,jb.perp(),wts);
	  // Plot the transverse momentum of the extremal partons
	  pi=event.getpartvn(0);
	  Fill2DHistograms(hptp1,hptp1N,hptp1wt2,pi.perp(),wts);
	  pi=event.getpartvn(np-1);
	  Fill2DHistograms(hptp2,hptp2N,hptp2wt2,pi.perp(),wts);

	  // std::vector<fastjet::PseudoJet> hjets = sorted_by_pt(ev.inclusive_jets(Flags.jetptmin));
	  jha.set(hjets.at(0).px(),hjets.at(0).py(),hjets.at(0).pz(),hjets.at(0).e());
	  jhb.set(hjets.at(1).px(),hjets.at(1).py(),hjets.at(1).pz(),hjets.at(1).e());
	  Fill2DHistograms(hpth1,hpth1N,hpth1wt2,jha.perp(),wts);
	  Fill2DHistograms(hpth2,hpth2N,hpth2wt2,jhb.perp(),wts);
	  Fill2DHistograms(hpthr,hpthrN,hpthrwt2,jhb.perp()/jha.perp(),wts);
	  Fill2DHistograms(hht,hhtN,hhtwt2,ht,wts);
	  double sij=2.*jha.dot(jhb);
	  //	  std::cout<<"sij = "<<sij<<"  "<<jha<<"  "<<jhb<<"\n";
	  Fill2DHistograms(hsij,hsijN,hsijwt2,sij,wts);

	  Fill2DHistograms(hptbar,hptbarN,hptbarwt2,ptbar,wts);
	  Fill2DHistograms(hnjets,hnjetsN,hnjetswt2,njetst,wts);
	  tphtjets->Fill(ht,njetst,wts);
	  tpsijjets->Fill(sij,njetst,wts);
	  // hnp->Fill(event.getnp(),wts);
	  Fill2DHistograms(hnp,hnpN,hnpwt2,event.getnp(),wts);
	  // havgnjets->Fill(yb-ya,njetst*wts*5.);
	  Fill2DHistograms(havgnjets,havgnjetsN,havgnjetswt2,fabs(yb-ya),njetst*wts);
	  // havgnjetsnorm->Fill(yb-ya,wts*5.);
	  tpavgjets->Fill(fabs(yb-ya),njetst,wts);
	  Fill2DHistograms(havgnjetsnorm,havgnjetsnormN,havgnjetsnormwt2,fabs(yb-ya),wts);
	  double cosphi=((ja.x()*jb.x()+ja.y()*jb.y())/(sqrt(ja.x()*ja.x()+ja.y()*ja.y())*sqrt(jb.x()*jb.x()+jb.y()*jb.y())));
	  // hcosphi->Fill(fabs(ya-yb),-4.*cosphi*wt);
	  Fill2DHistograms(hcosphi,hcosphiN,hcosphiwt2,fabs(ya-yb),-cosphi*wts);
	  // hcosphinorm->Fill(fabs(ya-yb),4.*wt);
	  Fill2DHistograms(hcosphinorm,hcosphinormN,hcosphinormwt2,fabs(ya-yb),wts);
	  tpavgcosphi->Fill(fabs(ya-yb),-cosphi,wts);
	  
	  // dsigma/dphi for various rapidity slices
	  Fill2DHistograms(hdsdphi1,hdsdphi1N,hdsdphi1wt2,acos(cosphi),wts);
	  if (fabs(ya-yb)>=1&&fabs(ya-yb)<2)
	    Fill2DHistograms(hdsdphi2,hdsdphi2N,hdsdphi2wt2,acos(cosphi),wts);
	  if (fabs(ya-yb)>=2&&fabs(ya-yb)<3)
	    Fill2DHistograms(hdsdphi3,hdsdphi3N,hdsdphi3wt2,acos(cosphi),wts);
	  if (fabs(ya-yb)>=3&&fabs(ya-yb)<4)
	    Fill2DHistograms(hdsdphi4,hdsdphi4N,hdsdphi4wt2,acos(cosphi),wts);

	  hwt->Fill(log(fabs(wt)));
	  hwtwt->Fill(log(fabs(wt)),fabs(wt));

	}
    }
  }
  // event.xsec[1][0]=htotal->GetBinContent(1);
  // event.xsec[1][1]=htotal->GetBinError(1);
  std::cout<<"Cross section passing cuts (mub): "<<htotal->GetBinContent(1)<<" +/- "<<htotal->GetBinError(1)<<std::endl;
  for (int count=1;count<=4;count++) 
    std::cout<<"Cross section passing cuts (mub): "<<event.xsec[count][0]<<" +/- "<<sqrt(event.xsec[count][1])<<std::endl;

  for (int count=1;count<=4;count++) 
    std::cout << event.maxweight[count] << " ";
  std::cout << endl;

  hfile.Write();
  hfile.Close();
  return 0;
}
